#!/bin/sh
#
#    Copyright (c) 2011-2014 Brocade Communications Systems, Inc.
#    All rights reserved.
#
#    File name:   cos_asic_ssave.sh.in
#    Module name: utils/support
#
#    This file expected to be invoked by supportsave infrastructure to
#    collect ASIC driver related data for switch post failure analysis
#

ssaveutil="/fabos/cliexec/ssave_util.sh"

#
# Include ssave utility - sets PATH, STANDBY, CAT, etc.
#
if [ -e "$ssaveutil" ]; then
    . $ssaveutil
    rc=$?
    if [ $rc -ne 0 ]; then
        echo "Error -- $ssaveutil err=$rc"
        exit 3
    fi
else
    echo "Error -- could not locate ssave common library $ssaveutil"
    exit 3
fi

# set -x

# Base dir
# /var is used to store output files before zipped/tarred
dir_base=/var

cmd_title()
{
	$ECHO
	$ECHO "########################################################"
	$ECHO "## $1"
	$ECHO "########################################################"
}

# Input: slot#
# Return:  If Ok, blade_type & blade_id is set properly for the slot
# Otherwise, bladeid ==0, and blade_type == ""
asic_db_blade_type_() {
	# initialize to unknown blade id & type
	bladeid=0
	blade_type=""

	if test -f /proc/fabos/blade/$1/info; then

		# ref. blade id is defined in linux/include/brocade/swbd.h
		bladeid=`$CAT /proc/fabos/blade/$1/info | /bin/grep --binary-files=text bladeId | sed 's/[^0-9]//g'`

		case "$bladeid" in
			"15" ) blade_type="pulsar-cosim";;  	# Pulsar
			"114" ) blade_type="mercury-48x10g";;   # MERCURY_LC48X10G_BLADE_ID
			* ) blade_type="others";; # pulsar/stealth/proton
		esac
	fi

	$ECHO "Slot: $1 blade id: $bladeid type: $blade_type"
	# return status
	if [ "$blade_type" = "" ]; then
		return 1
	else
		return 0
	fi
}

# Generic ASIC SS
# Walk all mini/chip combo to find the $asic_name
#
# input:
# asic_name		: chip name
# do_asic_cmds	: chip specific handler
#
start_asic_db_cmd()
{
	$ECHO "Start $asic_name supportsave"
	if [ "$STANDBY" == "TRUE" ]; then
		$ECHO "Local is in standby"
	fi

	# If db script not present, can't dump database
	#if [ ! -e /fabos/link_sbin/db ]; then
	str=`db 2>/dev/null`
	if [ -z "$str" ]; then
		$ECHO "Unable to locate db script."
		return
	fi

	export DB_SILENT_ERR=1

	# Base dir/file names
	# $dir_base is used to store output files before zipped/tarred
	# All output files will start with $asic_name
	# Per chip db commands output to xxdb.slot.mini.chip.txt
	# Per chip regdump output to xxregdump.slot.mini.chip.dmp
	# Per chip off-switch script file xxdmp.slot.mini.chip.cmd
	# Per chip tar ball xx.slot.mini.chip.tar.gz
	# All chips tar ball xxsupport.tar
	chp_base=$dir_base/$asic_name
	db_base=${chp_base}db
	regdmp_base=${asic_name}regdump
	offswutil_affix=_dboffswutil
	asicdmp_base=${chp_base}$offswutil_affix
	asic_tar_ball=/tmp/${asic_name}support.tar
	lpmdmp_base=${asic_name}lpmdump
	exmdmp_base=${asic_name}exmdump
	tcamdmp_base=${asic_name}tcamdump

	chp_cnt=0

	/bin/rm -f $db_base.*
	/bin/rm -f $chp_base.*
	/bin/rm -f $dir_base/$regdmp_base.*
	/bin/rm -f $asicdmp_base.*
	cd /proc/fabos/blade

	# Create the global tar ball intially to include the off-switch
	# decoder/compare/asicdump utilities
	# Each per chip tar ball will be added later in the for loop below.
	if [ -e /fabos/cliexec/${asic_name}_hregdecode ]; then
		/bin/cp /fabos/cliexec/${asic_name}_hregdecode $dir_base/.
		hregdec=$dir_base/${asic_name}_hregdecode
	else
		hregdec=$dir_base/no_${asic_name}_hregdecode.txt
		$ECHO "/fabos/cliexec/${asic_name}_hregdecode not found" > $hregdec
	fi
	if [ -e /fabos/cliexec/${asic_name}_hregcmp ]; then
		/bin/cp /fabos/cliexec/${asic_name}_hregcmp $dir_base/.
		hregcmp=$dir_base/${asic_name}_hregcmp
	else
		hregcmp=$dir_base/no_${asic_name}_hregcmp.txt
		$ECHO "/fabos/cliexec/${asic_name}_hregcmp not found" > $hregcmp
	fi
	if [ -e /fabos/cliexec/${asic_name}$offswutil_affix ]; then
		/bin/cp /fabos/cliexec/${asic_name}$offswutil_affix $dir_base/.
		asicdmp=$dir_base/${asic_name}$offswutil_affix
	else
		asicdmp=$dir_base/no_${asic_name}$offswutil_affix.txt
		$ECHO "/fabos/cliexec/${asic_name}$offswutil_affix not found" > $asicdmp
	fi
	# $ECHO "Doing: tar -cvf $asic_tar_ball $hregdec $hregcmp $asicdmp"
	$CHASSISCMD tar -cvf $asic_tar_ball $hregdec $hregcmp $asicdmp
	/bin/rm $hregdec
	/bin/rm $hregcmp
	/bin/rm $asicdmp

	for slot in `ls -dF1 [0-9]* | sort -n | grep / | sed 's;/;;'`; do
		asic_db_blade_type_ $slot
		if [ "$?" -eq 0 ]; then
			cd $slot
			$ECHO "Process Slot: $slot"
			for mini in `ls -dF1 [0-9]* | sort -n | grep / | sed 's;/;;'`; do
				cd $mini
				$ECHO "Process Slot: $slot Mini: $mini"
				# Per chip
				for chp in `ls -dF1 [0-9]* | sort -n | grep / | sed 's;/;;'`; do
					asic_chip_present $slot $mini $chp
					if [ "$?" -eq 0 ]; then
						$ECHO "Process Slot: $slot Mini: $mini Chip: $chp"

						db_out=$db_base.$slot.$mini.$chp.txt
						chp_out=$chp_base.$slot.$mini.$chp.tar.gz
						regdmp_out=$regdmp_base.$slot.$mini.$chp.dmp
						asicdmp_cmd=$asicdmp_base.$slot.$mini.$chp.cmd
						lpmdmp_out=$lpmdmp_base.$slot.$mini.$chp.dmp
						exmdmp_out=$exmdmp_base.$slot.$mini.$chp.dmp
						tcamdmp_out=$tcamdmp_base.$slot.$mini.$chp.dmp
						regdmp_out2=$regdmp_base.$slot.$mini.$chp.dmp2

						# Initial output file list, more added later
						out_list="$db_out"
						$ECHO
						date
						# $ECHO "Doing: $do_asic_cmds $slot $mini $chp > $db_out"
						$do_asic_cmds $slot $mini $chp > $db_out 2>&1
						if [ -n "$err_string" ]; then
							$ECHO "$do_asic_cmds $slot $mini $chp fails: $err_string"
						fi
						# Zip per chip files in a tar file.
						# $ECHO "Doing: tar -czvf $chp_out $out_list"
						$CHASSISCMD tar -czvf $chp_out $out_list
						/bin/rm $out_list
						# add this per chip to the global tar bal
						$CHASSISCMD tar -rvf $asic_tar_ball $chp_out
						/bin/rm $chp_out
						(( chp_cnt += 1 ))
					fi
				done
				# back to $slot
				cd ..
			done
			# back to /proc/fabos/blade
			cd ..
		fi
	done
	unset DB_SILENT_ERR
    
	do_wlv_aq_cmds

	$ECHO "Found $chp_cnt $asic_name chips on this switch."
	if [ $chp_cnt -eq 0 ]; then
		# No chips found so remove the tar ball and create a new one
		# with just a single line of text to indicate the same.
		/bin/rm $asic_tar_ball
		$ECHO "No $asic_name chips found on this switch" > $dir_base/${asic_name}emptyregdump.txt
		$CHASSISCMD tar -cvf $asic_tar_ball $dir_base/${asic_name}emptyregdump.txt
		/bin/rm $dir_base/${asic_name}emptyregdump.txt
	fi

	sync
	$ECHO ""
}

#
# Return 0 if slot/mini/chip is ASIC named in $4
#
# input:
# $1	: slot
# $2	: mini
# $3	: chip
# $4	: ASIC name
#
asic_db_present()
{
	db_path=$1/$2/$3
	debug_file=/proc/fabos/blade/$db_path/debug

	local chp_name=$4
	$ECHO "Scan for $chp_name at $db_path"
	if [ ! -e $debug_file ]; then
		# Return error
		return 1
	fi

	# Make sure dbg module is registered
	db $db_path | grep dbg > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		# Return error
		return 1
	fi

	# Not this asic if ${asic_name}_stat module is NOT registered
	db $db_path | grep ${chp_name}_stat > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		# Return error
		return 1
	fi

	db $db_path $TRC_BOOKMARK "SS_found_$db_path"
	# Looks like this slot/mini/chip is $asic_name
	# return 0
	$ECHO "Found $chp_name at $db_path"
}

#
# Return 0 if slot/mini/chip is $asic_name
#
# input:
# $1	: slot
# $2	: mini
# $3	: chip
#
# output:
# $chp_generation	: ${asic_name}x
#
asic_chip_present()
{
	chp_generation="$asic_name"
	if [ "$asic_name" = "hawk" ]; then
		# Hawk1 chip
		asic_db_present $1 $2 $3 "hawk"
		if [ "$?" -eq 0 ]; then
			chp_generation="hawk1"
			return 0
		fi

	elif [ "$asic_name" = "hawk2" ]; then
		# Hawk2 chip
		asic_db_present $1 $2 $3 "hawk2"
		if [ "$?" -eq 0 ]; then
			chp_generation="hawk2"
			return 0
		fi
	elif [ "$asic_name" = "wlv" ]; then
		# Wolverine chip
		# Try wlv1 first
		asic_db_present $1 $2 $3 "wlv"
		if [ "$?" -eq 0 ]; then
			chp_generation="wlv1"
			return 0
		fi

		# Then try wlv2
		asic_db_present $1 $2 $3 "wlv2"
		if [ "$?" -eq 0 ]; then
			chp_generation="wlv2"
			return 0
		fi
	else
		# Other chip (for future)
		# Should not get here as currently we only support hawk/wlv
		asic_db_present $1 $2 $3 "$asic_name"
		if [ "$?" -eq 0 ]; then
			return 0
		fi
	fi

	# No known chip found.
	return 1
}

#
# Check if hawk/wlv driver is ready for the db commands
#
# input
# $1	: full path slot/mini/chip
#
# output:
# $err_string	: error string if not ready.
#
# return 0 if ready
#
db_cmd_rdy()
{
	err_string=
	str=`db $1 dbg verify ss_db 2>/dev/null`
	if [ -z "$str" ]; then
		err_string="dbg command fails."
		$ECHO "$err_string"
		return 1
	fi

	# This string should match the one produced by {hawk,wlv}_ss_db_veri()
	# in {hawk,wlv}_init.c
	if [ "$str" = "SS DB cmd is ready" ]; then
		return 0
	fi

	if [ "$STANDBY" == "TRUE" ]; then
		err_string="Local is in standby"
	else
		err_string="ASIC driver is not ready for db commands"
	fi

	$ECHO "$err_string"
	return -1
}

#
# Per Hawk1/hawk2 chip SS
#
# input:
# $1	: slot
# $2	: mini
# $3	: chip
#
do_hawk_cmds()
{
	if [ "$chp_generation" = "hawk1" ]; then
		do_hawk1_cmds $1 $2 $3
	else
		do_hawk2_cmds $1 $2 $3
	fi
}

#
# Per Hawk chip SS
#
# input:
# $1	: slot
# $2	: mini
# $3	: chip
#
do_hawk1_cmds()
{
	proc_path=$1/$2/$3
	cmd_title "Hawk Chip $proc_path"
	db $db_path $TRC_BOOKMARK "SS_Hawk_${proc_path}_Begin"

	# Proceed only if the driver is ready for db commands
	db_cmd_rdy $proc_path
	if [ "$?" -ne 0 ]; then
		db $db_path $TRC_BOOKMARK "SS_Hawk_${proc_path}_not_ready"
		#
		# These commands don't access HW registers so they are safe
		# to run when in standby or before chip attach
		#
		cmd_title "Chip Information"
		$CHASSISCMD db $proc_path dbg show chip no_detail
		$CHASSISCMD db $proc_path dbg verify chip_init
		$ECHO ""
		cmd_title "Infrastructure Information"
		$CHASSISCMD db $proc_path infra show all
		$ECHO ""
		cmd_title "Stats Information"
		$CHASSISCMD db $proc_path hawk_stat dump stat blk inf
		$ECHO ""
		cmd_title "Error Infrastructure Information"
		$CHASSISCMD db $proc_path err show stats
		db $db_path $TRC_BOOKMARK "SS_Hawk_${proc_path}_End"
		return
	fi

	# Add off-switch util command file.
	out_list="$out_list $asicdmp_cmd"

	#
	# PCIe TBUF information
	#
	cmd_title "PCIe TBUF Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_PCIe_TBUF_info"
	$CHASSISCMD db $proc_path tbuf capture show
	$CHASSISCMD db $proc_path tbuf show reg
	$CHASSISCMD db $proc_path tbuf show stats
	$ECHO ""

	#
	# Chip information
	#
	cmd_title "Chip Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_CHIP_Info"
	$CHASSISCMD db $proc_path dbg show chip no_detail
	$CHASSISCMD db $proc_path dbg verify chip_init
	$CHASSISCMD db $proc_path cfg show chip
	$CHASSISCMD db $proc_path cfg show chp_cap
	$CHASSISCMD db $proc_path cfg show e2e
	$ECHO ""

	#
	# Phyp information
	#
	cmd_title "Phyp Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Phyp_Info"
	$CHASSISCMD db $proc_path phyp show ports
	$ECHO ""

	#
	# Reg dump
	#
	cmd_title "Regdump Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_Info"
	$CHASSISCMD regdump $proc_path
	if [ -e /var/regdump.dmp ]; then
		mv /var/regdump.dmp $dir_base/$regdmp_out
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$regdmp_out"
	else
		$ECHO "regdump $proc_path fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_fails"
	fi
	$ECHO ""

	#
	# Generate off-switch asicdump script file for this regdump
	# This file can be used to feed into the off-switch asicdump util
	#
	# First command should be: load regdump
	$ECHO "db hawk dbg load regdump $regdmp_out" > $asicdmp_cmd

	#
	# Stats
	#
	cmd_title "Stats Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Stats_Info"
	# Dump from HW, but only if regdump failed
	if ! $ECHO $out_list | grep -q $dir_base/$regdmp_out ; then
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Stats_all"
		$CHASSISCMD db $proc_path hawk_stat dump stat all
	fi
	# Stats Buffer dump - captures same info as cache dump
	$CHASSISCMD do_asic_blk_dump statsbuf $1 $2 $3 hawk_stat dump_bufs
	# Add "load statsbuf" command to the off-switch util script
	if [ "$?" -eq 0 ]; then
		$ECHO "db hawk dbg load statsbuf $dump_file" >> $asicdmp_cmd
		$ECHO "db $proc_path hawk_stat dump stat all" >> $asicdmp_cmd
		$ECHO "db $proc_path hawk_stat dump pb_stat all" >> $asicdmp_cmd
	fi
	$ECHO "db $proc_path ccb show icb red_voq" >> $asicdmp_cmd
	for ((j = 0; j < 33; j++)); do
		$ECHO "db $proc_path ccb show ecb port $j" >> $asicdmp_cmd
	done
	for ((j = 0; j < 15; j++)); do
		$ECHO "db $proc_path ccb show ecb red_prf $j" >> $asicdmp_cmd
	done

	$ECHO "db $proc_path sch show eoq glb" >> $asicdmp_cmd
	for ((j = 0; j < 33; j++)); do
		$ECHO "db $proc_path sch show eoq port $j" >> $asicdmp_cmd
	done

	$ECHO "db $proc_path sch show ivq glb" >> $asicdmp_cmd
	for ((j = 0; j < 48; j++)); do
		$ECHO "db $proc_path sch show ivq_chip $j" >> $asicdmp_cmd
		for ((k = 0; k < 34; k++)); do
			$ECHO "db $proc_path sch show ivq_chip_port $j $k" >> $asicdmp_cmd
		done
	done


	$ECHO ""

	#
	# Infrastructure
	#
	cmd_title "Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Infra_Info"
	$CHASSISCMD db $proc_path infra show all
	$ECHO ""

	#
	# Error Infrastructure
	#
	cmd_title "Error Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Err_infra_Info"
	$CHASSISCMD db $proc_path err show stats
	$CHASSISCMD db $proc_path err show err_table
	$CHASSISCMD db $proc_path err show cfg_table
	$CHASSISCMD db $proc_path err show xlate_table
	$ECHO ""

	#
	# HA Infrastructure
	#
	cmd_title "HA Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_HA_Infra_Info"
	$CHASSISCMD db $proc_path ha show clients
	$CHASSISCMD db $proc_path ha show database
	$CHASSISCMD db $proc_path ha show hbuf
	$ECHO ""

	#
	# txrx dump
	#
	cmd_title "TXRX Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_TXRX_Info"
	$CHASSISCMD db $proc_path txrx show info
	$CHASSISCMD db $proc_path txrx show stats all
	$CHASSISCMD db $proc_path txrx show hib

	$CHASSISCMD db $proc_path txrx show rxq stats
	$CHASSISCMD db $proc_path txrx show txq all
	$CHASSISCMD db $proc_path txrx show txcredit all

	for ((j = 0; j < 5; j++)); do
		$CHASSISCMD db $proc_path txrx show info $j
	done

	for ((j = 0; j < 5; j++)); do
		$CHASSISCMD db $proc_path txrx dump desc $j all
	done

	for ((j = 0; j < 5; j++)); do
		$CHASSISCMD db $proc_path txrx show desc $j 0
	done

	db $db_path $TRC_BOOKMARK "SS_${proc_path}_TXRX_pcap"
	$CHASSISCMD db $proc_path txrx pcap dump state
	$CHASSISCMD db $proc_path txrx pcap dump all
	$ECHO ""

	#
	# FMC
	#
	cmd_title "FMC Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_FMC_Info"
	$CHASSISCMD db $proc_path fmc show kr
	$ECHO ""

	#
	# E2E
	#
	cmd_title "E2E Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_E2E_Info"
	$CHASSISCMD db $proc_path e2e show supportsave
	$ECHO ""

	#
	# IME/EME
	#
	cmd_title "IME/EME Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_IME_EME_Info"
	$CHASSISCMD db $proc_path mcast show supportsave
	$ECHO ""

	#
	# ICB
	#
	cmd_title "ICB Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_ICB_Info"
	$CHASSISCMD db $proc_path ccb show icb glbbuf
	$ECHO ""
	for ((j = 0; j < 37; j++)); do
		$ECHO icb port $j
		$CHASSISCMD db $proc_path ccb show icb port $j
	done
	# skip "ccb icb_stat_tbl show", output is too large, not that interesting
	$CHASSISCMD db $proc_path ccb show_ha glb
	for ((j = 0; j < 37; j++)); do
		$CHASSISCMD db $proc_path ccb show_ha port $j
	done
	$ECHO ""

	#
	# ECB stuff all done off-switch
	# EOQ stuff all done off-switch
	# IVQ stuff all done off-switch
	#

	# Put this last
	# Add "quit" command at the end of off-switch util script
	$ECHO "quit" >> $asicdmp_cmd
	db $db_path $TRC_BOOKMARK "SS_Hawk_${proc_path}_End"

	#
	# 2nd Reg dump
	#
	cmd_title "Regdump 2nd Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_2_Info"
	$CHASSISCMD regdump $proc_path
	if [ -e /var/regdump.dmp ]; then
		mv /var/regdump.dmp $dir_base/$regdmp_out2
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$regdmp_out2"
	else
		$ECHO "regdump $proc_path fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_fails"
	fi
	$ECHO ""
}

#
# Per Hawk2 chip SS
#
# input:
# $1	: slot
# $2	: mini
# $3	: chip
#
do_hawk2_cmds()
{
	proc_path=$1/$2/$3
	cmd_title "Hawk2 Chip $proc_path"
	db $db_path $TRC_BOOKMARK "SS_Hawk2_${proc_path}_Begin"

	IMPLEMENTED=0
	TODO=1

	# Proceed only if the driver is ready for db commands
	db_cmd_rdy $proc_path
	if [ "$?" -ne 0 ]; then
		db $db_path $TRC_BOOKMARK "SS_Hawk2_${proc_path}_not_ready"
		#
		# These commands don't access HW registers so they are safe
		# to run when in standby or before chip attach
		#
		cmd_title "Chip Information"
		$CHASSISCMD db $proc_path dbg show chip no_detail
		$CHASSISCMD db $proc_path dbg verify chip_init
		$ECHO ""
		cmd_title "Infrastructure Information"
		$CHASSISCMD db $proc_path infra show all
		$ECHO ""
		cmd_title "Stats Information"
		$CHASSISCMD db $proc_path hawk_stat dump stat blk inf
		$ECHO ""
		cmd_title "Error Infrastructure Information"
		$CHASSISCMD db $proc_path err show stats
		db $db_path $TRC_BOOKMARK "SS_Hawk2_${proc_path}_End"
		return
	fi

	# Add off-switch util command file.
	out_list="$out_list $asicdmp_cmd"

	#
	# Chip voltage and temperature
	#
	cmd_title "Chip Voltage and Temperature Information"
	$CHASSISCMD db $proc_path dbg show temp
	$ECHO ""

	# TBUF db cli is not implemented
if [ $IMPLEMENTED -eq 1 ]; then
	#
	# PCIe TBUF information
	#
	cmd_title "PCIe TBUF Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_PCIe_TBUF_info"
	$CHASSISCMD db $proc_path tbuf capture show
	$CHASSISCMD db $proc_path tbuf show reg
	$CHASSISCMD db $proc_path tbuf show stats
	$ECHO ""
fi

	#
	# Chip information
	#
	cmd_title "Chip Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_CHIP_Info"
	$CHASSISCMD db $proc_path dbg show chip no_detail
	$CHASSISCMD db $proc_path dbg verify chip_init
	$CHASSISCMD db $proc_path cfg show chip
	$CHASSISCMD db $proc_path cfg show chp_cap
	$CHASSISCMD db $proc_path cfg show e2e
	$ECHO ""

	#
	# Phyp information
	#
	cmd_title "Phyp Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Phyp_Info"
	$CHASSISCMD db $proc_path phyp show ports
	$ECHO ""

	#
	# Reg dump
	#
	cmd_title "Regdump Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_Info"
	$CHASSISCMD regdump $proc_path
	if [ -e /var/regdump.dmp ]; then
		mv /var/regdump.dmp $dir_base/$regdmp_out
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$regdmp_out"
	else
		$ECHO "regdump $proc_path fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_fails"
	fi
	$ECHO ""

	#
	# Generate off-switch asicdump script file for this regdump
	# This file can be used to feed into the off-switch asicdump util
	#
	# First command should be: load regdump
	$ECHO "db hawk dbg load regdump $regdmp_out" > $asicdmp_cmd

	#
	# Stats
	#
	cmd_title "Stats Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Stats_Info"
	cmd_title "MSG port Stats Information"
	for ((j = 30; j < 32; j++)); do
		$CHASSISCMD db $proc_path rstats dump fmc_port $j
	done
	# Dump from HW, but only if regdump failed
	if ! $ECHO $out_list | grep -q $dir_base/$regdmp_out ; then
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Stats_all"
		$CHASSISCMD db $proc_path hawk2_stat dump stat all
	fi
	# Stats Buffer dump - captures same info as cache dump
	$CHASSISCMD do_asic_blk_dump statsbuf $1 $2 $3 hawk2_stat dump_bufs
	# Add "load statsbuf" command to the off-switch util script
	if [ "$?" -eq 0 ]; then
		$ECHO "db hawk2 dbg load statsbuf $dump_file" >> $asicdmp_cmd
		$ECHO "db $proc_path hawk2_stat dump stat all" >> $asicdmp_cmd
		$ECHO "db $proc_path hawk2_stat dump pb_stat all" >> $asicdmp_cmd
	fi

	for ((j = 0; j < 24; j++)); do
		$ECHO "db $proc_path ccb show ecb port $j" >> $asicdmp_cmd
	done
	for ((j = 0; j < 16; j++)); do
		$ECHO "db $proc_path ccb show ecb red_prf $j" >> $asicdmp_cmd
	done

	$ECHO "db $proc_path sch show eoq glb" >> $asicdmp_cmd
	for ((j = 0; j < 25; j++)); do
		$ECHO "db $proc_path sch show eoq port $j" >> $asicdmp_cmd
	done

	$ECHO "db $proc_path sch show ivq glb" >> $asicdmp_cmd
	for ((j = 0; j < 64; j++)); do
		$ECHO "db $proc_path sch show ivq_chip $j" >> $asicdmp_cmd
		for ((k = 0; k < 26; k++)); do
			$ECHO "db $proc_path sch show ivq_chip_port $j $k" >> $asicdmp_cmd
		done
	done

	$ECHO ""

	#
	# Infrastructure
	#
	cmd_title "Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Infra_Info"
	$CHASSISCMD db $proc_path infra show all
	$ECHO ""

	#
	# Error Infrastructure
	#
	cmd_title "Error Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Err_infra_Info"
	$CHASSISCMD db $proc_path err show stats
	$CHASSISCMD db $proc_path err show err_table
	$CHASSISCMD db $proc_path err show cfg_table
	$CHASSISCMD db $proc_path err show xlate_table
	$ECHO ""

	#
	# HA Infrastructure
	#
	cmd_title "HA Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_HA_Infra_Info"
	$CHASSISCMD db $proc_path ha show clients
	$CHASSISCMD db $proc_path ha show database
	$CHASSISCMD db $proc_path ha show hbuf
	$ECHO ""

	#
	# txrx dump
	#
	cmd_title "TXRX Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_TXRX_Info"
	$CHASSISCMD db $proc_path txrx show info
	$CHASSISCMD db $proc_path txrx show stats all
	$CHASSISCMD db $proc_path txrx show hib

	$CHASSISCMD db $proc_path txrx show rxq stats
	$CHASSISCMD db $proc_path txrx show txq all
	$CHASSISCMD db $proc_path txrx show txcredit all

	for ((j = 0; j < 5; j++)); do
		$CHASSISCMD db $proc_path txrx show info $j
	done

	for ((j = 0; j < 5; j++)); do
		$CHASSISCMD db $proc_path txrx dump desc $j all
	done

	for ((j = 0; j < 5; j++)); do
		$CHASSISCMD db $proc_path txrx show desc $j 0
	done

	# db $db_path $TRC_BOOKMARK "SS_${proc_path}_TXRX_pcap"
	# $CHASSISCMD db $proc_path txrx pcap dump state
	# $CHASSISCMD db $proc_path txrx pcap dump all
	$ECHO ""

	#
	# FMC
	#
	cmd_title "FMC Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_FMC_Info"
	$CHASSISCMD db $proc_path fmc show kr
	$ECHO ""

	#
	# E2E
	#
	cmd_title "E2E Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_E2E_Info"
	$CHASSISCMD db $proc_path e2e show supportsave
	$ECHO ""

	#
	# IME/EME
	#
	cmd_title "IME/EME Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_IME_EME_Info"
	$CHASSISCMD db $proc_path mcast show supportsave
	$ECHO ""

	#
	# ICB
	#
	cmd_title "ICB Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_ICB_Info"
	$CHASSISCMD db $proc_path ccb show icb glbbuf
	$ECHO ""
	for ((j = 0; j < 37; j++)); do
		$ECHO icb port $j
		$CHASSISCMD db $proc_path ccb show icb port $j
	done
	# skip "ccb icb_stat_tbl show", output is too large, not that interesting
	$CHASSISCMD db $proc_path ccb show_ha glb
	for ((j = 0; j < 37; j++)); do
		$CHASSISCMD db $proc_path ccb show_ha port $j
	done
	$ECHO ""

	#
	# ECB stuff all done off-switch
	# EOQ stuff all done off-switch
	# IVQ stuff all done off-switch
	#
	# Put this last
	# Add "quit" command at the end of off-switch util script
	$ECHO "quit" >> $asicdmp_cmd
	db $db_path $TRC_BOOKMARK "SS_Hawk2_${proc_path}_End"

	#
	# 2nd Reg dump
	#
	cmd_title "Regdump 2nd Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_2_Info"
	$CHASSISCMD regdump $proc_path
	if [ -e /var/regdump.dmp ]; then
		mv /var/regdump.dmp $dir_base/$regdmp_out2
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$regdmp_out2"
	else
		$ECHO "regdump $proc_path fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_fails"
	fi
}

#
# Per WLV1/WLV2 chip SS
#
# input
# $1	: slot
# $2	: mini
# $3	: chip
#
do_wlv_cmds()
{
	if [ "$chp_generation" = "wlv1" ]; then
		do_wlv1_cmds $1 $2 $3
	else
		do_wlv2_cmds $1 $2 $3
	fi
}

#
# Per Wolverine chip SS
#
# input
# $1	: slot
# $2	: mini
# $3	: chip
#
do_wlv1_cmds()
{
	proc_path=$1/$2/$3
	cmd_title "Wolverine Chip $proc_path"
	db $db_path $TRC_BOOKMARK "SS_WLV_${proc_path}_Begin"

	# Proceed only if the driver is ready for db commands
	db_cmd_rdy $proc_path
	if [ "$?" -ne 0 ]; then
		db $db_path $TRC_BOOKMARK "SS_WLV_${proc_path}_not_ready"
		#
		# These commands don't access HW registers so they are safe
		# to run when in standby or before chip attach
		#
		cmd_title "Chip Information"
		$CHASSISCMD db $proc_path dbg show chip no_detail
		$CHASSISCMD db $proc_path dbg verify chip_init
		$ECHO ""
		cmd_title "Infrastructure Information"
		$CHASSISCMD db $proc_path infra show all
		$ECHO ""
		cmd_title "Stats Information"
		$CHASSISCMD db $proc_path wlv_stat dump stat blk inf
		$ECHO ""
		cmd_title "Error Infrastructure Information"
		$CHASSISCMD db $proc_path err show stats
		db $db_path $TRC_BOOKMARK "SS_WLV_${proc_path}_End"
		return
	fi

	# Add off-switch util command file.
	out_list="$out_list $asicdmp_cmd"

	#
	# Chip voltage and temperature
	#
	cmd_title "Chip Voltage and Temperature Information"
	$CHASSISCMD db $proc_path dbg show temp
	$ECHO ""

	#
	# PCIe TBUF information
	#
	cmd_title "PCIe TBUF Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_PCIe_TBUF_info"
	$CHASSISCMD db $proc_path tbuf capture show
	$CHASSISCMD db $proc_path tbuf show reg
	$CHASSISCMD db $proc_path tbuf show stats
	$ECHO ""

	#
	# Chip information
	#
	cmd_title "Chip Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_CHIP_Info"
	$CHASSISCMD db $proc_path dbg show chip no_detail
	$CHASSISCMD db $proc_path dbg verify chip_init
	$CHASSISCMD db $proc_path cfg show chip
	$CHASSISCMD db $proc_path cfg show chp_cap
	# For now just do the first 8 ports
	# Fix this for future support of more than 8.
	for ((j = 0; j < 8; j++)); do
		$CHASSISCMD db $proc_path cfg show pt $j
	done
	$ECHO ""

	#
	# Phyp information
	#
	cmd_title "Phyp Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Phyp_Info"
	$CHASSISCMD db $proc_path phyp show ports
	$ECHO ""

	#
	# Reg dump
	#
	cmd_title "Regdump Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_Info"
	$CHASSISCMD regdump $proc_path
	if [ -e /var/regdump.dmp ]; then
		mv /var/regdump.dmp $dir_base/$regdmp_out
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$regdmp_out"
	else
		$ECHO "regdump $proc_path fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_fails"
	fi
	$ECHO ""

	#
	# LPM dump
	#
	cmd_title "Lpmdump Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Lpmdump_Info"
	$CHASSISCMD asicdump $proc_path lpm
	if [ -e /var/lpmdump.dmp ]; then
		mv /var/lpmdump.dmp $dir_base/$lpmdmp_out
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$lpmdmp_out"
	else
		$ECHO "asicdump $proc_path lpm fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Lpmdump_Fails"
	fi
	$ECHO ""

	#
	# EXM dump
	#
	cmd_title "Exmdump Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Exmdump_Info"
	$CHASSISCMD asicdump $proc_path exm
	if [ -e /var/exmdump.dmp ]; then
		mv /var/exmdump.dmp $dir_base/$exmdmp_out
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$exmdmp_out"
	else
		$ECHO "asicdump $proc_path exm fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Exmdump_Fail"
	fi
	$ECHO ""

	#
	# TCAM dump
	#
	cmd_title "Tcamdump Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Tcamdump_Info"
	$CHASSISCMD asicdump $proc_path tcam
	if [ -e /var/tcamdump.dmp ]; then
		mv /var/tcamdump.dmp $dir_base/$tcamdmp_out
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$tcamdmp_out"
	else
		$ECHO "asicdump $proc_path tcam fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Tcamdump_Fail"
	fi
#
#	$ECHO ""
	#
	# Generate off-switch asicdump script file for this regdump
	# This file can be used to feed into the off-switch asicdump util
	#
	# First command should be: load regdump
	$ECHO "db wlv dbg load regdump $regdmp_out" > $asicdmp_cmd

	# Put all modules supported commands from here
	$ECHO "db wlv rte show capture" >> $asicdmp_cmd
	$ECHO "db wlv dbg peek rte_his_rslt -max 100" >> $asicdmp_cmd
	$ECHO "db wlv ccb show all" >> $asicdmp_cmd
	$ECHO "db wlv sch show all" >> $asicdmp_cmd

	#
	# LAG
	# 
	cmd_title "Link Aggregation (LAG)  Information"
	$ECHO "db wlv lag show flavor all" >> $asicdmp_cmd
	$ECHO "db wlv lag show outputs all" >> $asicdmp_cmd
	$ECHO "db wlv lag show stats assignment" >> $asicdmp_cmd
	$ECHO "db wlv lag show ingress table -1 -1" >> $asicdmp_cmd
	$ECHO "db wlv lag show egress table -1" >> $asicdmp_cmd

	#
	# Stats
	#
	cmd_title "Stats Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Stats_Info"
	# Dump from HW, but only if regdump failed
	if ! $ECHO $out_list | grep -q $dir_base/$regdmp_out ; then
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Stats_all"
		$CHASSISCMD db $proc_path wlv_stat dump stat all one_row
	fi
	# Stats Buffer dump - captures same info as cache dump
	$CHASSISCMD do_asic_blk_dump statsbuf $1 $2 $3 wlv_stat dump_bufs
	# Add "load statsbuf" command to the off-switch util script
	if [ "$?" -eq 0 ]; then
		$ECHO "db wlv dbg load statsbuf $dump_file" >> $asicdmp_cmd
		$ECHO "db $proc_path wlv_stat dump stat all one_row" >> $asicdmp_cmd
		$ECHO "db $proc_path wlv_stat dump pb_stat all one_row" >> $asicdmp_cmd
	fi
	$ECHO ""

	#
	# ICB/ECB
	#
	cmd_title "ICB/ECB"
	$ECHO "db $proc_path ccb show icb glb" >> $asicdmp_cmd
	$ECHO "db $proc_path ccb show icb log" >> $asicdmp_cmd
	$ECHO "db $proc_path ccb show icb pib" >> $asicdmp_cmd
	$ECHO "db $proc_path ccb show icb elb" >> $asicdmp_cmd
	$ECHO "db $proc_path ccb show icb others" >> $asicdmp_cmd
	$ECHO "db $proc_path ccb show ecb glb" >> $asicdmp_cmd
	$ECHO "db $proc_path ccb show ecb log" >> $asicdmp_cmd
	for lgc in `seq 0 31` ; do
		$ECHO "db $proc_path ccb show icb port $lgc" >> $asicdmp_cmd
		$ECHO "db $proc_path ccb show ecb port $lgc" >> $asicdmp_cmd
	done;
	for phy in `seq 0 11` ; do
		$ECHO "db $proc_path ccb show ingress maps all phy $phy" >> $asicdmp_cmd
	done;

	#
	# IQS/EQS
	#
	cmd_title "IQS/EQS"
	$ECHO "db $proc_path sch show iqs glb" >> $asicdmp_cmd
	$ECHO "db $proc_path sch show iqs bp" >> $asicdmp_cmd
	$ECHO "db $proc_path sch show iqs others" >> $asicdmp_cmd
	$ECHO "db $proc_path sch show eqs glb" >> $asicdmp_cmd
	$ECHO "db $proc_path sch show eqs bp" >> $asicdmp_cmd
	for lgc in `seq 0 33` ; do
		$ECHO "db $proc_path sch show iqs port $lgc" >> $asicdmp_cmd
	done;
	for lgc in `seq 0 31` ; do
		$ECHO "db $proc_path sch show eqs port $lgc" >> $asicdmp_cmd
	done;

	$CHASSISCMD db $proc_path sch show eqs qdepth
	$ECHO ""

	#
	# Infrastructures
	#
	cmd_title "Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Infra_Info"
	$CHASSISCMD db $proc_path infra show all
	$ECHO ""

	#
	# Error Infrastructure
	#
	cmd_title "Error Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Err_infra_Info"
	$CHASSISCMD db $proc_path err show stats
	$CHASSISCMD db $proc_path err show err_table
	$CHASSISCMD db $proc_path err show cfg_table
	$CHASSISCMD db $proc_path err show xlate_table
	$ECHO ""

	#
	# HA Infrastructure
	#
	cmd_title "HA Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_HA_Infra_Info"
	$CHASSISCMD db $proc_path ha show clients
	$CHASSISCMD db $proc_path ha show database
	$CHASSISCMD db $proc_path ha show hbuf
	$ECHO ""

	#
	# MAC refresh dump
	#
	cmd_title "MAC Refresh Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_MAC_RFSH_Info"
	$CHASSISCMD db $proc_path util show mac_refresh ss
	$ECHO ""

	#
	# txrx dump
	#
	cmd_title "TXRX Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_TXRX_Info"
	$CHASSISCMD db $proc_path txrx show info
	$CHASSISCMD db $proc_path txrx show stats all
	$CHASSISCMD db $proc_path txrx show pib

	$CHASSISCMD db $proc_path txrx show rxq stats
	$CHASSISCMD db $proc_path txrx show txq all

	$CHASSISCMD db $proc_path txrx show txcredit all

	for ((j = 0; j < 6; j++)); do
		$CHASSISCMD db $proc_path txrx dump desc $j all
	done

	for ((j = 0; j < 6; j++)); do
		$CHASSISCMD db $proc_path txrx show desc $j 0
	done

	$CHASSISCMD db $proc_path txrx dump ka_pkt
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_TXRX_pcap"
	$CHASSISCMD db $proc_path txrx pcap dump state
	$CHASSISCMD db $proc_path txrx pcap dump all
	$ECHO ""

	#
	# TSQ/HSQ
	#
	cmd_title "TSQ/HSQ Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_TSQ_HSQ_Info"
	$CHASSISCMD db $proc_path trunk show all eport
	$ECHO ""

	#
	# Ethernet Loopback (ELB)
	#
	cmd_title "ELB Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_ELB_Info"
	$CHASSISCMD db $proc_path elb show status
	$ECHO ""

	#
	# mcast
	#
	cmd_title "Mcast Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Mcast_Info"
	$CHASSISCMD db $proc_path mcast show supportsave
	$ECHO ""

	#
	# LAG
	#
	cmd_title "LAG Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_LAG_Info"
	$CHASSISCMD db $proc_path lag show local all
	$ECHO ""

    #
    # QoS is mostly done off-switch, see above
    #
	for phy in `seq 0 11` ; do
		$CHASSISCMD db $proc_path ccb show cache phy $phy
		$CHASSISCMD db $proc_path sch show cache phy $phy
    done;

	#
	# POL
	#
	cmd_title "POL Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_POL_Info"
	$CHASSISCMD db $proc_path policer show all
	$ECHO ""

	#
	# FCoE
	#
	cmd_title "FCoE Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_FCoE_Info"
	$CHASSISCMD db $proc_path fcoe show basic
    # can be done offline:
    #    show ivid filter 0 0x2000
    #    show vfid filter 0 0x2000
    #    show nhop 0x1f000 0x8000 (best guess where entries typically go)
    #    show sa_mac 0 500
    #    show capture filters
    # rte items, fcoe related, done offline:
	#    rte show l2damac 0 500

    #
    # FCoE TCAM regions (zero entries actually means "all")
    #
#	cmd_title "FCoE L2 ACL"
#	db $db_path $TRC_BOOKMARK "SS_${proc_path}_FCoE_l2_ACL"
#	feature=`$CHASSISCMD db $proc_path rte show tcam psel region list | grep LDEV_L2PSEL_FCOE | awk '{print $3}'`
#	for region in `seq 0 9` ; do
#		$CHASSISCMD db $proc_path rte show tcam psel region entries $feature $region 0 0
#	done;

#	cmd_title "FCoE TCAM Forwarding Routes"
#	db $db_path $TRC_BOOKMARK "SS_${proc_path}_FCoE_TCAM_FwdRt"
#	feature=`$CHASSISCMD db $proc_path rte show tcam lpm region list | grep LDEV_L3FWD_FCOE | awk '{print $3}'`
#	for region in 0 1 2 ; do
#		$CHASSISCMD db $proc_path rte show tcam lpm region entries $feature $region 0 0
#	done;

#	cmd_title "FCoE L3 ACL"
#	db $db_path $TRC_BOOKMARK "SS_${proc_path}_FCoE_l3_ACL"
#	feature=`$CHASSISCMD db $proc_path rte show tcam psel region list | grep LDEV_L3PSEL_FCOE | awk '{print $3}'`
#	for region in 0 4 5 6 ; do
#		$CHASSISCMD db $proc_path rte show tcam psel region entries $feature $region 0 0
#	done;
    # fcoe show nhop can be taken from offline utility

	#
	# TRILL, related to FCoE
	#
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_FCoE_TRILL"
	$CHASSISCMD db $proc_path rte show mytrillrb
	$CHASSISCMD db $proc_path rte show trillrb 0 100
	$CHASSISCMD db $proc_path rte show l2_mpath 0 100
	$CHASSISCMD db $proc_path rte show span
	$CHASSISCMD db $proc_path rte show dyn_vrb 0 8192
	$CHASSISCMD db $proc_path rte show lpm nhop mac_idx

	#
	# RTE, EXM, TCAM
	cmd_title "RTE EXM Statistics"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_RTE_EXM_Stats"
	$CHASSISCMD db $proc_path rte show exm stats

#	cmd_title "RTE TCAM cfg Information"
#	db $db_path $TRC_BOOKMARK "SS_${proc_path}_RTE_TCAM_CFG"
#	$CHASSISCMD db $proc_path rte show tcam psel cfg
#	$CHASSISCMD db $proc_path rte show tcam eacl cfg
#	$CHASSISCMD db $proc_path rte show tcam lpm cfg
#	$CHASSISCMD db $proc_path rte show tcam mac cfg
#	$CHASSISCMD db $proc_path rte show tcam cld cfg
#	$CHASSISCMD db $proc_path rte show tcam ipv4mcu cfg
#	$CHASSISCMD db $proc_path rte show tcam ipv6mcu cfg
#	$CHASSISCMD db $proc_path rte show tcam l3ctrl cfg
#	$CHASSISCMD db $proc_path rte show tcam type cfg

#	cmd_title "RTE TCAM region Information"
#	db $db_path $TRC_BOOKMARK "SS_${proc_path}_RTE_TCAM_Info"
#	$CHASSISCMD db $proc_path rte show tcam psel region list
#	$CHASSISCMD db $proc_path rte show tcam eacl region list
#	$CHASSISCMD db $proc_path rte show tcam lpm region list
#	$CHASSISCMD db $proc_path rte show tcam mac region list
#	$CHASSISCMD db $proc_path rte show tcam cld region list
#	$CHASSISCMD db $proc_path rte show tcam ipv4mcu region list
#	$CHASSISCMD db $proc_path rte show tcam ipv6mcu region list
#	$CHASSISCMD db $proc_path rte show tcam l3ctrl region list
#	$CHASSISCMD db $proc_path rte show tcam type region list

	cmd_title "RTE TCAM dma refresh Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_TCAM_Refresh_Info"
	$CHASSISCMD db $proc_path rte show dma_refresh

	$ECHO ""

	#
	# XPHY STATUS - only for SWBD1006
	#
	if sin | grep -q SWBD1006 ; then
		cmd_title "External PHY Information"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Ext_Phy_Info"
		$CHASSISCMD db $proc_path xphy show -ss
		$ECHO
	fi

	# Put this last
	# Add "quit" command at the end of off-switch util script
	$ECHO "quit" >> $asicdmp_cmd
	db $db_path $TRC_BOOKMARK "SS_WLV_${proc_path}_End"

	#
	# 2nd Reg dump (need not to be part of off-switch script)
	#
	cmd_title "Regdump 2nd Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_2_Info"
	$CHASSISCMD regdump $proc_path
	if [ -e /var/regdump.dmp ]; then
		mv /var/regdump.dmp $dir_base/$regdmp_out2
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$regdmp_out2"
	else
		$ECHO "regdump $proc_path fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_2_fails"
	fi
	$ECHO ""
}

#
# Per Wolverine2 chip SS
#
# input
# $1	: slot
# $2	: mini
# $3	: chip
#
do_wlv2_cmds()
{
	proc_path=$1/$2/$3
	cmd_title "Wolverine2 Chip $proc_path"
	db $db_path $TRC_BOOKMARK "SS_WLV2_${proc_path}_Begin"
	err_string="SS on Wolverine2 is not supported yet"
	$ECHO "$err_string"
}

do_wlv_aq_cmds()
{
    slot=0
	mini=0
	chp=1

	for slot in `ls -dF1 [0-9]* | sort -n | grep / | sed 's;/;;'`; do
        asic_db_blade_type_ $slot
       	#
        # XPHY STATUS - only for SWBD1011
        #
        if [ $bladeid -eq 151 ] ; then
		    cmd_title "AQPHY Quad Information"
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 0 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 1 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 2 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 3 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 4 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 5 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 6 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 7 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 8 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 9 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 10 -quad 0
		    $CHASSISCMD db $slot/$mini/$chp xphy show -status -chip 11 -quad 0
		    port=1
		    fe_port=48
		    while [ $port -le $fe_port ]; do
			    cmd_title "AQPHY PORT $port Information"
			    $CHASSISCMD db $slot/$mini/$chp xphy show -reg -mmd 1 -port $port
			    $CHASSISCMD db $slot/$mini/$chp xphy show -reg -mmd 3 -port $port
			    $CHASSISCMD db $slot/$mini/$chp xphy show -reg -mmd 7 -port $port
			    $CHASSISCMD db $slot/$mini/$chp xphy show -stats -port $port -reg
			    $ECHO
			    (( port += 1 ))
		    done
		    $ECHO
       fi
    done
}

#
# Per Cobra chip SS
#
# input
# $1	: slot
# $2	: mini
# $3	: chip
#
do_cbr_cmds()
{
	proc_path=$1/$2/$3
	cmd_title "Cobra Chip $proc_path"
	db $db_path $TRC_BOOKMARK "SS_CBR_${proc_path}_Begin"

	# Proceed only if the driver is ready for db commands
	db_cmd_rdy $proc_path
	if [ "$?" -ne 0 ]; then
		db $db_path $TRC_BOOKMARK "SS_WLV_${proc_path}_not_ready"
		#
		# These commands don't access HW registers so they are safe
		# to run when in standby or before chip attach
		#
		cmd_title "Chip Information"
		$CHASSISCMD db $proc_path dbg show chip no_detail
		$CHASSISCMD db $proc_path dbg verify chip_init
		$ECHO ""
		cmd_title "Infrastructure Information"
		$CHASSISCMD db $proc_path infra show all
		$ECHO ""
		cmd_title "Stats Information"
		$CHASSISCMD db $proc_path wlv_stat dump stat blk inf
		$ECHO ""
		cmd_title "Error Infrastructure Information"
		$CHASSISCMD db $proc_path err show stats
		db $db_path $TRC_BOOKMARK "SS_WLV_${proc_path}_End"
		return
	fi

	#
	# Chip voltage and temperature
	#
	cmd_title "Chip voltage and temperature Information"
	# Read voltage
	# 0x00008480:  0x00000f8a  pib_tvsense_cntl
	# tvsense_snsout 0xF8= 248 (Voltage = 248*1.8617+506 =967.7016 mV )
	$CHASSISCMD db $proc_path dbg poke pib_tvsense_cntl 0x3
	$CHASSISCMD db $proc_path dbg poke pib_tvsense_cntl 0x2
	$CHASSISCMD db $proc_path dbg peek pib_tvsense_cntl

	# Read temperature
	# 0x00008480:  0x00001028  pib_tvsense_cntl
	# tvsense_snsout =0x102= 258 (Temp = 258 * 0.5 -90 = 39 C)
	$CHASSISCMD db $proc_path dbg poke pib_tvsense_cntl 0x1
	$CHASSISCMD db $proc_path dbg poke pib_tvsense_cntl 0x0
	$CHASSISCMD db $proc_path dbg peek pib_tvsense_cntl
	$ECHO ""

	#
	# PCIe TBUF information
	#
	cmd_title "PCIe TBUF Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_PCIe_TBUF_info"
	$CHASSISCMD db $proc_path tbuf capture show
	$CHASSISCMD db $proc_path tbuf show reg
	$CHASSISCMD db $proc_path tbuf show status
	$ECHO ""

	#
	# Chip information
	#
	cmd_title "Chip Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_CHIP_Info"
	$CHASSISCMD db $proc_path dbg show chip no_detail
	$CHASSISCMD db $proc_path dbg verify chip_init
	$CHASSISCMD db $proc_path cfg show chip
	$CHASSISCMD db $proc_path cfg show chp_cap
	for ((j = 0; j < 64; j++)); do
		$CHASSISCMD db $proc_path cfg show pt $j
	done
	$ECHO ""

	#
	# Phyp information
	#
	cmd_title "Phyp Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Phyp_Info"
	$CHASSISCMD db $proc_path phyp show ports
	$CHASSISCMD db $proc_path phyp show serdes
	$ECHO ""

	#
	# Reg dump
	#
	cmd_title "Regdump Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_Info"
	$CHASSISCMD regdump $proc_path
	if [ -e /var/regdump.dmp ]; then
		mv /var/regdump.dmp $dir_base/$regdmp_out
		# Add this dump file to the SS file list
		out_list="$out_list $dir_base/$regdmp_out"
	else
		$ECHO "regdump $proc_path fails"
		db $db_path $TRC_BOOKMARK "SS_${proc_path}_Regdump_fails"
	fi
	$ECHO ""

	#
	# Error Infrastructure
	#
	cmd_title "Error Infrastructure Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Err_infra_Info"
	$CHASSISCMD db $proc_path err show stats
	$CHASSISCMD db $proc_path err show err_table
	$CHASSISCMD db $proc_path err show cfg_table
	$CHASSISCMD db $proc_path err show xlate_table
	$ECHO ""

	#
	# Stats
	#
	cmd_title "Stats Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Stats_Info"
	$CHASSISCMD db $proc_path cbr_stat dump stat blk all
	$CHASSISCMD db $proc_path cbr_stat dump stat blk inf
	$CHASSISCMD db $proc_path cbr_stat dump drop_cnt blk all detail	
	$ECHO ""

	#
	# Trunking
	#
	cmd_title "Trunking Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Trunk_Info"
	$CHASSISCMD db $proc_path trunk show basic
	$CHASSISCMD db $proc_path trunk show all
	$ECHO ""

	#
	# RTE
	#
	cmd_title "RTE Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_RTE_Info"
	$CHASSISCMD db $proc_path rte show exm l2 summary
	$CHASSISCMD db $proc_path rte show exm ipv4mc summary
	$CHASSISCMD db $proc_path rte show exm ipv6mc summary
	$CHASSISCMD db $proc_path rte show exm ipv4mcu summary
	$CHASSISCMD db $proc_path rte show exm ipv6mcu summary
	$CHASSISCMD db $proc_path rte show exm ipv4addr summary
	$CHASSISCMD db $proc_path rte show exm ipv6addr summary
	$CHASSISCMD db $proc_path rte show exm fcoe summary
	$CHASSISCMD db $proc_path rte show exm fcoe used
	$CHASSISCMD db $proc_path rte show exm plcyid used
	$CHASSISCMD db $proc_path rte show exm ipv4mc used

	$CHASSISCMD db $proc_path rte show mpid mct cfg
	$CHASSISCMD db $proc_path rte show mpid drb cfg
	$CHASSISCMD db $proc_path rte show mpid lag cfg

	$CHASSISCMD db $proc_path rte show trill myrb
	$CHASSISCMD db $proc_path rte show tcam psel cfg
	$CHASSISCMD db $proc_path rte show tcam lpm cfg
	$CHASSISCMD db $proc_path rte show tcam l3ctrl cfg
	$CHASSISCMD db $proc_path rte show tcam type cfg

	$CHASSISCMD db $proc_path rte show span
	$ECHO ""

	#
	# MCAST Commands
	#
	cmd_title "Mcast Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_Mcast_Info"
	$CHASSISCMD db $proc_path mcast show mgid
	$CHASSISCMD db $proc_path mcast show pet all ipv4
	$CHASSISCMD db $proc_path mcast show pet all ipv6
	$CHASSISCMD db $proc_path mcast show l2_l3_assoc
	$CHASSISCMD db $proc_path mcast show l3_l2_chain_head
	$CHASSISCMD db $proc_path mcast show l3_l2_assoc
	$CHASSISCMD db $proc_path mcast show bmaprefcnt
	$ECHO ""

	#
	# DMA Commands
	#
	cmd_title "Tx/Rx Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_TXRX_Info"
	$CHASSISCMD db $proc_path txrx show pib control
	$CHASSISCMD db $proc_path txrx show pib errors
	$CHASSISCMD db $proc_path txrx show txcredit all
	$CHASSISCMD db $proc_path txrx show txq all
	$CHASSISCMD db $proc_path txrx show rxq stats
	$CHASSISCMD db $proc_path txrx show stats all
	$CHASSISCMD db $proc_path txrx show stats all active
	$CHASSISCMD db $proc_path txrx show info

	#
	# Dump KA queued packets
	#
	$CHASSISCMD db $proc_path txrx dump ka_pkt
	$CHASSISCMD db $proc_path txrx dump desc 22 0
	$CHASSISCMD db $proc_path txrx show port_info
	$CHASSISCMD db $proc_path txrx dump desc 1 0
	$CHASSISCMD db $proc_path txrx dump desc 0 0
	$CHASSISCMD db $proc_path txrx dump desc 19 0
	$CHASSISCMD db $proc_path txrx dump desc 20 0
	$CHASSISCMD db $proc_path txrx dump desc 21 0
	
	for ((j = 0; j < 64; j++)); do
		$CHASSISCMD db $proc_path txrx show trap stats $j
	done
	$ECHO ""

	#
	# LAG
	#
	cmd_title "LAG Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_LAG_Info"
	$CHASSISCMD db $slot/$chip/0 lag show local all
	$CHASSISCMD db $slot/$chip/0 lag show flavor
	$CHASSISCMD db $slot/$chip/0 lag show egress table -1
	$CHASSISCMD db $slot/$chip/0 lag show ingress table all
	$ECHO ""

	#
	# FCoE commands
	#
	cmd_title "FCoE Information"
	db $db_path $TRC_BOOKMARK "SS_${proc_path}_FCoE_Info"
	$CHASSISCMD db $slot/$chip/0 fcoe show basic
	$ECHO ""
}

#
# Generic ASIC block's buffer dump.
#
# full dump file name:
# dump_file=${chp_base}_$1.slot.mini.chip.dmp
#
# input
# $1	: base dump file name
# $2	: slot
# $3	: mini
# $4	: chip
# $5	: db module name to initiate the dump
# $6	: db command name to initiate the dump
#
# output:
# out_list="$out_list $dump_file"
#
do_asic_blk_dump()
{
	# Does the chip exist?
	chip=$1
	if [ ! -e /proc/fabos/blade/$2/$3/$4 ]; then
		$ECHO "Chip $2/$3/$4 not in /proc"
		return 1
	fi

	# Trigger ASIC block buffer dump
	cmd="db $2/$3/$4 $5 $6"
	# $ECHO "Doing $cmd"
	str=`$cmd`

	if [ -z "$str" ]; then
		$ECHO "db command fails: $cmd"
		return 1
	fi

	# Extract buffer address and size from string:
	#   "kernel addr: 0x68d01000 size: 0x300000"
	addr=`echo $str | grep kernel | cut -f3 -d" "`
	size=`echo $str | grep kernel | cut -f5 -d" "`

	if [ -z "$addr" ] || [ -z "$size" ]; then
		$ECHO "Unexpected output from: $cmd"
		return 1
	fi

	if [ $((addr)) -eq 0 ]; then
		$ECHO "Invalid addr: $addr"
		return 1
	fi

	if [ $((size)) -eq 0 ]; then
		$ECHO "Invalid size: $size"
		return 1
	fi

	#
	# Copy kernel buffer to file
	#
	page_size_bit=12
	page_size=$((1 << page_size_bit))
	blocks=$(((size + page_size - 1) / page_size))
	offset=$((addr >> page_size_bit))

	# Bash 2.4.0 does not support unsigned integer. Need to mask off "sign
	# extended" bits for the offset.
	mask=$((0x7fffffff >> (page_size_bit - 1)))
	offset=$((offset & mask))

	dump_file="${asic_name}_$1.$2.$3.$4.dmp"
	dd if=/dev/kmem of=$dir_base/$dump_file count=$blocks bs=$page_size skip=$offset &> /dev/nul

	if [ $? -ne 0 ]; then
		$ECHO "Fails to dump $5 $6"
		return 1
	fi

	if [ ! -e $dir_base/$dump_file ]; then
		$ECHO "Fails to create dump file $dir_base/$dump_file"
		return 1
	fi

	$ECHO "$cmd dumped to $dir_base/$dump_file"
	# Add this dump file to the SS file list
	out_list="$out_list $dir_base/$dump_file"

	return 0
}

# *******************************************
# Collect supportsave
# *******************************************
start_support()
{
	fname=${1##*--}
	/bin/touch -c /tmp/$fname.tar
	if [ ! -f /tmp/$fname.tar ]; then
		$ECHO "File /tmp/$fname.tar not found"
		return 255
	fi

	$ECHO "File /tmp/$fname.tar info:"
	/bin/ls -l /tmp/$fname.tar

	return 0
}

# ***** Main ****************************************
export PATH=/fabos/link_bin:/bin:/usr/bin:/sbin:/usr/sbin:/fabos/link_abin:/fabos/link_sbin:/fabos/bin

	not_yet="echo not yet "

	# add xx_TRACE_BOOKMARK trace entry
	TRC_BOOKMARK="dbg bookmark trace ASIC_MISC"

	for param in $@; do
		if [ "$param" = "--hawkdb" ]; then
			do_asic_cmds="do_hawk1_cmds"
			asic_name=hawk
			group_show start_asic_db_cmd
			RETVAL=`$ECHO $?`
		fi

		if [ "$param" = "--hawk2db" ]; then
			do_asic_cmds="do_hawk2_cmds"
			asic_name=hawk2
			group_show start_asic_db_cmd
			RETVAL=`$ECHO $?`
		fi

		if [ "$param" = "--wlvdb" ]; then
			do_asic_cmds="do_wlv_cmds"
			asic_name=wlv
			group_show start_asic_db_cmd
			RETVAL=`$ECHO $?`
		fi

		if [ "$param" = "--cbrdb" ]; then
			do_asic_cmds="do_cbr_cmds"
			asic_name=cbr
			group_show start_asic_db_cmd
			RETVAL=`$ECHO $?`
		fi

		if [ "$param" = "--hawksupport" ] || [ "$param" = "--wlvsupport" ] ||
			[ "$param" = "--hawk2support" ] || [ "$param" = "--cbrsupport" ]; then
			start_support $param
			RETVAL=`$ECHO $?`
		fi

		exit $RETVAL
	done
# ***** End of Main *****
